/*= R&S SMV Vector Signal Generator =========================================*/
/* LabWindows/CVI 6.0 VXIpnp Instrument Driver                               */
/* Original Release: December 2002                                           */
/* By: Martin Koutny                                                         */
/*                                                                           */
/* Should you have any technical questions please contact the hotline of     */
/* Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center             */
/*                                                                           */
/* e-mail: CustomerSupport@rsd.rohde-schwarz.com                             */
/*                                                                           */
/* Modification History:                                                     */
/*                                                                           */
/*   1.0.3, 02/2008, P.Marcalik                                              */
/*     new functions                                                         */
/*      rssmv_setSweepBlankTime                                              */
/*      rssmv_powerSweepPerformed                                            */
/*      rssmv_getFreqCorrPoints                                              */
/*      rssmv_setAllModulationState                                          */
/*      rssmv_utilSystemKeylock                                              */
/*                                                                           */
/*   1.0.2, 04/2004, Jiri Kominek                                            */ 
/*    Added CHM Help file                                                    */ 
/*    Fixed floating point precision problem                                 */
/*                                                                           */
/*   1.0.1, 01/2004, Jiri Kominek                                            */ 
/*    Modifications                                                          */ 
/*    For Agilent VISA (Ver. M01.01) and Serial Interface only               */
/*       - changed rssmv_init                                                */     
/*                                                                           */                                                                           
/*===========================================================================*/
#include <visa.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "rssmv.h"

#define RSSMV_REVISION     "Rev 1.0.3, 02/2008, CVI 8.1"  /* Instrument driver revision */
#define BUFFER_SIZE         512L                        /* File I/O buffer size */
#define IEEE_ERROR_BITS     0x3C

/*= Macros ==================================================================*/
#define CHECKERR(fCal) if (rssmv_status = (fCal), rssmv_status < VI_SUCCESS) \
    return rssmv_status; else

#define CHECKERR_CU(fCal) if (rssmv_status = (fCal), rssmv_status < VI_SUCCESS) \
    return rssmv_initCleanUp (rmSession, instrSession, rssmv_status); else

/* Check presency of instrument's option (when error checking is enabled) */
#define CHECKOPT(option) if (rssmv_status = (instrPtr -> instrOption & (option)) ? \
    VI_SUCCESS : (instrPtr -> errorChecking) ? RSSMV_ERROR_INSTRUMENT_OPTION : VI_SUCCESS, \
    rssmv_status < VI_SUCCESS) \
    return rssmv_status; else

/* Instrument Options */
#define RSSMV_OPT_B1        (1 << 0)
/* reserved                 (1 << 1) */
#define RSSMV_OPT_B3        (1 << 2)
/* reserved                 (1 << 3) */
#define RSSMV_OPT_B5        (1 << 4)
/* reserved                 (1 << 5) */
#define RSSMV_OPT_B19       (1 << 6)

/* Instrument Models */
#define RSSMV_SMV01         (1 << 0)
#define RSSMV_SMV02         (1 << 1)
#define RSSMV_SMV03         (1 << 2)

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/
static ViString freqResetArr[] = {"INCL", "EXCL"};
static ViString RFFreqModeArr[] = {"FIX", "SWE", "LIST"};  
static ViString SweepModeArr[] = {"AUTO", "MAN", "STEP"};
static ViString RFFreqSweepSpacArr[] = {"LIN","LOG"};
static ViString RFAttModeArr[] = {"AUTO", "FIX"};
static ViString RFOutPonSArr[] = {"OFF", "UNCH"};
static ViString AMSourceArr[] = {"INT", "EXT", "TTON"};
static ViString AMExtCoupArr[] = {"AC", "DC"};
static ViString FMBandArr[] = {"STAN","WIDE"};
static ViString PolarityArr[] = {"NORM", "INV"};
static ViString PMSourceArr[] = {"EXT", "INT"};
static ViString LFFreqSweepSpacArr[] = {"LIN", "LOG"};
static ViString pulseOutSourceArr[] = {"OFF", "PULS", "VID"};
static ViString RFOscSourceArr[] = {"INT", "EXT"};
static ViString RFOscRLoopArr[] = {"NORM", "NARR"};
static ViString TriggerSourceArr[] = {"SING", "EXT", "AUTO"};
static ViString PulseTriggerSourceArr[] = {"AUTO","EXT","EGAT"};
static ViString PulseTriggerSlopeArr[] = {"POS", "NEG"};
static ViString TriggerArr[] = {"SWE", "LIST", "MSEQ"};
static ViString CalArr[] = {":CAL:LPR",":CAL:LFG",":CAL:HARM",":CAL:MULT",
                            ":CAL:IFF",":CAL:MAIN",":CAL:FMOF",":CAL:ALL",
                            ":CAL:VMOD",":SOUR:POW:ALC:TABL"};
static ViString DisplayParamArr[] = {"ALL", "AMPL", "FREQ"};
static ViString BaudRateArr[] = {"1200","2400","4800","9600","19200","38400",
                                 "57600","115200"};
static ViString softHandArr[] = {"XON", "NONE"};
static ViString ParityArr[] = {"ODD","EVEN","NONE"};
static ViString hardHandArr[] = {"ON", "IBF", "RFR", ""};
static ViString TestArr[] = {"RAM", "ROM", "BATT"};
static ViString RegisterArr[] = {":STAT:OPER:EVEN",":STAT:OPER:COND",
                                 ":STAT:OPER:PTR",":STAT:OPER:NTR",
                                 ":STAT:OPER:ENAB",":STAT:QUES:EVEN",
                                 ":STAT:QUES:COND",":STAT:QUES:PTR",
                                 ":STAT:QUES:NTR",":STAT:QUES:ENAB",
                                 "*ESE","*ESR","*SRE","*STB"};
static ViString stereoSourceArr[] = {"LREX","SPEX","LFG"};
static ViString stereoModeArr[] = {"LEFT","RIGH","REL","REML","RNEL"};
static ViString stereoImpedanceArr[] = {"600Ohm","100kOhm"};
static ViString pinstrSession[] = {":STAT OFF"," 50"," 75"};
static ViString ARIIdentificationArr[] = {":STAT OFF"," DK"," BK"," BKDK"};
static ViString BKCodeArr[] = {"A","B","C","D","E","F"};
static ViString RDSDataSetArr[] = {"DS1","DS2","DS3","DS4","DS5"};
static ViString triggerPolarityArr[] = {"NORM","INV"};
static ViString sweepBTimeArr[] = {"NORM", "LONG"}; 
static ViString sweepPerformedArr[] = {"SOUR:SWE:RUNN","SOUR2:SWE:RUNN","SOUR:SWE:POW:RUNN"};  

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* rssmv_stringValPair is used in the rssmv_error_message function           */
/* rssmv_statusDataRanges is used to track session dependent status & ranges */
/*===========================================================================*/
typedef struct  rssmv_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  rssmv_tStringValPair;

struct rssmv_statusDataRanges {

    ViInt32     instrModel;
    ViInt32     instrOption;
    ViBoolean   errorChecking;
    ViChar      instrDriverRevision[256];
};

typedef struct rssmv_statusDataRanges *rssmv_instrRange;

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViBoolean rssmv_invalidViBooleanRange (ViBoolean val);
ViBoolean rssmv_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean rssmv_invalidViReal64Range (ViReal64 val, ViReal64 min,
                ViReal64 max);
ViBoolean rssmv_invalidViString (ViString val, ViUInt32 maxLength);
ViStatus rssmv_initCleanUp (ViSession openRMSession,
                ViPSession openInstrSession,
                ViStatus currentStatus);
ViStatus rssmv_defaultInstrSetup (ViSession openInstrSession);
ViStatus rssmv_checkFreqRange (ViSession instrSession, ViReal64 frequency, 
                ViReal64 min, ViStatus error);
ViStatus rssmv_checkFreqStep (ViSession instrSession, ViReal64 step, 
                ViReal64 min, ViStatus error);

/*****************************************************************************/
/*------ INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINE DECLARATIONS HERE ------*/
/*****************************************************************************/
ViStatus rssmv_instrOptions (ViSession instrSession, ViPInt32 options);
ViStatus rssmv_checkStatus (ViSession instrSession);

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_init (ViRsrc resourceName, ViBoolean IDQuery,
                    ViBoolean resetDevice, ViPSession instrSession)
{
    ViStatus    rssmv_status = VI_SUCCESS;
    ViUInt16    interface = VI_INTF_GPIB;
    ViSession   rmSession = 0;
    ViChar      rdBuffer[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/
    if (rssmv_invalidViBooleanRange (IDQuery))
        return VI_ERROR_PARAMETER2;
    if (rssmv_invalidViBooleanRange (resetDevice))
        return VI_ERROR_PARAMETER3;

    /*- Open instrument session ---------------------------------------------*/
    CHECKERR( viOpenDefaultRM (&rmSession));

    if ((rssmv_status = viOpen (rmSession, resourceName, VI_LOAD_CONFIG,
                                 VI_NULL, instrSession)) < 0)
        {
        viClose (rmSession);
        return rssmv_status;
        }

    /* Serial interface support */
    CHECKERR_CU( viGetAttribute (*instrSession, VI_ATTR_INTF_TYPE, &interface));

    if (interface == VI_INTF_ASRL)  /* Serial Port */
        {
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_TERMCHAR, '\n'));
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE));
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR));
        /* Changed to VI_ASRL_END_NONE due to Agilent VISA */
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_ASRL_END_OUT, VI_ASRL_END_NONE));
        CHECKERR_CU( viFlush (*instrSession, VI_ASRL_IN_BUF));
        
        CHECKERR_CU( viPrintf (*instrSession, "\n\n"));
        }
        
    /*- Configure VISA Formatted I/O ----------------------------------------*/
    CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 10000));
    CHECKERR_CU( viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000));
    CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
                                 VI_FLUSH_ON_ACCESS));
    CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
                                 VI_FLUSH_ON_ACCESS));

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery)
        {
        CHECKERR_CU( viQueryf (*instrSession, "*IDN?\n", "%s", rdBuffer));
        if (!strstr (rdBuffer, "ROHDE&SCHWARZ,SMV"))
            return rssmv_initCleanUp (rmSession, instrSession,
                                       VI_ERROR_FAIL_ID_QUERY);
        }       

    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice)
        rssmv_status = rssmv_reset (*instrSession);
    else  /*- Send Default Instrument Setup ---------------------------------*/
        rssmv_status = rssmv_defaultInstrSetup (*instrSession);
    CHECKERR_CU( rssmv_status);

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Output Ampl Modulated Signal                                    */
/* Purpose:  This function configures RF frequency output, switches it on    */
/*           and applies AM modulation on it with selected parameters.       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_exOutAmpl (ViSession instrSession, ViReal64 frequency,
                                   ViReal64 offset, ViReal64 level,
                                   ViInt32 source, ViReal64 depth,
                                   ViReal64 AMFrequency)
{
    ViStatus rssmv_status = VI_SUCCESS;
    
    CHECKERR (rssmv_checkFreqRange (instrSession, frequency, 9.0e3, VI_ERROR_PARAMETER2));
    
    if (rssmv_invalidViReal64Range (offset, -50.0e9, 50.0e9))
        return VI_ERROR_PARAMETER3;
    if (rssmv_invalidViReal64Range (level, -140.0, 25.0))
        return VI_ERROR_PARAMETER4;
    if (rssmv_invalidViInt32Range (source,0,2))  
       return VI_ERROR_PARAMETER5;
    if (rssmv_invalidViReal64Range (depth, 0.0, 100.0))  
       return VI_ERROR_PARAMETER6;
    if (rssmv_invalidViReal64Range (AMFrequency, 0.1, 1.0e6))
       return VI_ERROR_PARAMETER7;

    /* Switches state checking to off. This will speed-up performance of the bus. */
    CHECKERR (rssmv_errorCheckState (instrSession, VI_FALSE));
    
    CHECKERR (rssmv_configRFFreq (instrSession, frequency, offset, 1.0));
    CHECKERR (rssmv_configRFLevel (instrSession, level, 0.0, level));
    CHECKERR (rssmv_configAMModulation (instrSession, source, depth, AMFrequency));
    CHECKERR (rssmv_setRFOutputState (instrSession, VI_TRUE));
    
    /* Switches state checking to on. */
    CHECKERR (rssmv_errorCheckState (instrSession, VI_TRUE));

    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Configure RF Frequency                                          */
/* Purpose:  This function sets the frequency and offset.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_configRFFreq (ViSession instrSession,
                                      ViReal64 frequency, ViReal64 offset,
                                      ViReal64 multiplier)
{
    ViStatus rssmv_status = VI_SUCCESS;
    
    CHECKERR (rssmv_checkFreqRange (instrSession, frequency, 9.0e3, VI_ERROR_PARAMETER2));

    if (rssmv_invalidViReal64Range (offset, -50.0e9, 50.0e9))
        return VI_ERROR_PARAMETER3;
    if (rssmv_invalidViReal64Range (multiplier, 1.0, 10.0))
        return VI_ERROR_PARAMETER4;

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:OFFS 0.0;MULT 1.0;MODE CW;"
                        ":SOUR:FREQ %Lf;:SOUR:FREQ:OFFS %Lf;MULT %Le\n",
                        frequency, offset, multiplier));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set CW Frequency                                                */
/* Purpose:  This function sets the frequency for CW operation               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setCWFreq (ViSession instrSession, ViReal64 frequency)
{                               
    ViStatus rssmv_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ %Lf\n", frequency));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Frequency Offset                                         */
/* Purpose:  This function sets the frequency offset of a mixer.             */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFFreqOffset (ViSession instrSession,
                                         ViReal64 offset)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (offset, -50.0e9, 50.0e9))
        return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:OFFS %Lf\n", offset));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}
                                         
/*===========================================================================*/
/* Function: Set RF Frequency Multiplier                                     */
/* Purpose:  This function sets the frequency multiplier.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFFreqMultiplier (ViSession instrSession,
                                             ViReal64 multiplier)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (multiplier, 1.0, 10.0))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:MULT %Le\n", multiplier));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set CW Freq Reset                                               */
/* Purpose:  This function determines the effect of the recall               */
/*           function on the frequency.                                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setCWFreqReset (ViSession instrSession,
                                        ViInt32 resetState)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (resetState, 0, 1))    
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:RCL %s\n", freqResetArr[resetState]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Frequency Mode                                           */
/* Purpose:  This function specifies the RF operating mode.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFFreqMode (ViSession instrSession, ViInt32 mode)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (mode, 0, 1))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:MODE %s\n", RFFreqModeArr[mode]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Configure RF Frequency Sweep                                    */
/* Purpose:  This function sets the sweep mode, start and stop frequency.    */
/*           increment and dwell time.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_configRFFreqSweep (ViSession instrSession, ViInt32 mode,
                                           ViReal64 startFrequency,
                                           ViReal64 stopFrequency,
                                           ViReal64 increment, ViReal64 dwellTime)
{
    ViStatus rssmv_status = VI_SUCCESS;

    ViReal64    freqMin = 10.0e6,
                freqMax = 60.0e9;

    /* Query actual MIN and MAX values */
    CHECKERR (viQueryf (instrSession, ":SOUR:FREQ? MIN;FREQ? MAX\n", "%Le;%Le",
        &freqMin, &freqMax));

    if (rssmv_invalidViInt32Range (mode, 0, 2))  
       return VI_ERROR_PARAMETER2;
    if (rssmv_invalidViReal64Range (startFrequency, freqMin, freqMax))   
       return VI_ERROR_PARAMETER3;
    if (rssmv_invalidViReal64Range (stopFrequency, freqMin, freqMax))   
       return VI_ERROR_PARAMETER4;

    CHECKERR (rssmv_checkFreqStep (instrSession, increment, 0.0, VI_ERROR_PARAMETER5));

    if (rssmv_invalidViReal64Range (dwellTime, 0.01, 5.0))   
       return VI_ERROR_PARAMETER6;                              

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:FREQ:MODE %s;:SOUR:FREQ:STAR %Lf"
        ";STOP %Lf;:SOUR:FREQ:STEP:INCR %Lf;:SOUR:SWE:DWEL %Lf\n",
        SweepModeArr[mode], startFrequency, stopFrequency, increment,
        dwellTime));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Freq Sweep Mode                                          */
/* Purpose:  This function specifies specifies the run of the sweep.         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFFreqSweepMode (ViSession instrSession,
                                            ViInt32 mode)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (mode, 0, 2))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:FREQ:MODE %s\n", SweepModeArr[mode]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Start Frequency                                          */
/* Purpose:  This function defines the starting value of the frequency       */
/*           for the sweep operation.                                        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFStartFreq (ViSession instrSession,
                                        ViReal64 frequency)
{                               
    ViStatus rssmv_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:STAR %Lf\n", frequency));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Stop Frequency                                           */
/* Purpose:  This function defines the final value of the frequency          */
/*           for the sweep operation.                                        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFStopFreq (ViSession instrSession,
                                       ViReal64 frequency)
{                               
    ViStatus rssmv_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:STOP %Lf\n", frequency));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Frequency Increment                                      */
/* Purpose:  This function defines the step width for the frequency setting. */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFFreqIncrement (ViSession instrSession,
                                            ViReal64 increment)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    CHECKERR (rssmv_checkFreqStep (instrSession, increment, 0.0, VI_ERROR_PARAMETER2));  

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:STEP:INCR %Lf\n", increment));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Freq Sweep Dwell Time                                    */
/* Purpose:  This function defines the dwell time per frequency step.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFFreqSweepDwellTime (ViSession instrSession,
                                                 ViReal64 time)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (time, 0.01, 5.0))    
       return VI_ERROR_PARAMETER2;                              

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:DWEL %Lf\n", time));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Freq Sweep Spacing                                       */
/* Purpose:  This function defines whether the steps have linear             */
/*           or logarithmic spacings.                                        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFFreqSweepSpacing (ViSession instrSession,
                                               ViInt32 spacing)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (spacing, 0, 1))   
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:FREQ:SPAC %s\n",
        RFFreqSweepSpacArr[spacing]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Freq Sweep Step                                          */
/* Purpose:  This function defines the step width for linear                 */
/*           and logarithmic sweeps                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFFreqSweepStep (ViSession instrSession,
                                            ViInt32 mode, ViReal64 step)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (mode, 0, 1))
       return VI_ERROR_PARAMETER2;

    if (mode == 0)
        {
        if (rssmv_invalidViReal64Range (step, 0.0, 1.0e9))   
           return VI_ERROR_PARAMETER3;
        }
    else
        {
        if (rssmv_invalidViReal64Range (step, 0.01, 10.0))   
           return VI_ERROR_PARAMETER3;
        }
                                                                   
    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:FREQ:STEP:%s %Lf\n",
        RFFreqSweepSpacArr[mode], step));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Sweep Frequency Manual                                   */
/* Purpose:  This function the manual frequency.                             */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFSweepFreqManual (ViSession instrSession,
                                              ViReal64 frequency)
{                               
    ViStatus rssmv_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:MAN %Lf\n", frequency));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Center Frequency.                                        */
/* Purpose:  This function sets the sweep range by means                     */
/*           of the center frequency.                                        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFCenterFreq (ViSession instrSession,
                                         ViReal64 frequency)
{                               
    ViStatus rssmv_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:CENT %Lf\n", frequency));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Span Frequency.                                          */
/* Purpose:  This function sets the the frequency range for the sweep.       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFSpanFreq (ViSession instrSession,
                                       ViReal64 frequency)
{                               
    ViStatus rssmv_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:SPAN %Lf\n", frequency));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Extended Divider Range                                   */
/* Purpose:  This function sets activates or deactivates the Extended Divider*/
/*           Range function.                                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFExtDividerRange (ViSession instrSession,
                                              ViBoolean state)
{
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViBooleanRange (state))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:FREQ:ERAN %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Sweep Blank Time                                            */
/* Purpose:  This function defines the blank time of the sweep.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setSweepBlankTime (ViSession instrSession,
                                           ViInt32 time)
{                               
    ViStatus rssmv_status = VI_SUCCESS;

    if (rssmv_invalidViInt32Range (time, 0, 1))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:BTIM %s\n", sweepBTimeArr[time]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Configure RF Level.                                             */
/* Purpose:  This function sets the RF level, offset and limit.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_configRFLevel (ViSession instrSession, ViReal64 level,
                                       ViReal64 offset, ViReal64 limit)
{
    ViStatus rssmv_status = VI_SUCCESS;

    if (rssmv_invalidViReal64Range (level, -140.0, 25.0))    
       return VI_ERROR_PARAMETER2;
    if (rssmv_invalidViReal64Range (offset, -100.0, 100.0))  
       return VI_ERROR_PARAMETER3;
    if (rssmv_invalidViReal64Range (limit, -140.0, 25.0))    
       return VI_ERROR_PARAMETER4;

    CHECKERR (viPrintf (instrSession, ":POW:OFFS 0.0;:POW:LIM MAX;:POW %Lf;"
                        ":POW:OFFS %Lf;:POW:LIM %Lf\n", level, offset, limit));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Level.                                                   */
/* Purpose:  This function sets the RF output level in operating mode CW.    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFLevel (ViSession instrSession, ViReal64 level)
{                               
    ViStatus rssmv_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, ":POW %Lf\n", level));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Level Offset                                             */
/* Purpose:  This function sets the constant level offset.                   */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFLevelOffset (ViSession instrSession,
                                          ViReal64 offset)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (offset, -100.0, 100.0))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":POW:OFFS %Lf\n", offset));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Level Limit                                              */
/* Purpose:  This function sets the maximum RF output level.                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFLevelLimit (ViSession instrSession, ViReal64 limit)
{                               
    ViStatus rssmv_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, ":POW:LIM %Lf\n", limit));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Level Mode                                               */
/* Purpose:  This function specifies the RF operating mode.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFLevelMode (ViSession instrSession, ViInt32 mode)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (mode, 0, 1))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":POW:MODE %s\n", RFFreqModeArr[mode]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Level Reset                                              */
/* Purpose:  This function determines the effect of the recall               */
/*           function on the RF level.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFLevelReset (ViSession instrSession,
                                         ViInt32 resetState)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (resetState, 0, 1))    
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:POW:RCL %s\n", freqResetArr[resetState]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Configure RF Level Sweep.                                       */
/* Purpose:  This function sets the RF level, offset and limit.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_configRFLevelSweep (ViSession instrSession,
                                            ViInt32 mode, ViReal64 startLevel,
                                            ViReal64 stopLevel, ViReal64 step,
                                            ViReal64 dwellTime)
{
    ViStatus rssmv_status = VI_SUCCESS;
    ViReal64    powMin = -140.0,
                powMax = 25.0;

    /* Query actual MIN and MAX values */
    CHECKERR (viQueryf (instrSession, ":SOUR:POW? MIN;POW? MAX\n", "%Le;%Le",
        &powMin, &powMax));

    if (rssmv_invalidViInt32Range (mode, 0, 2))  
       return VI_ERROR_PARAMETER2;
    if (rssmv_invalidViReal64Range (startLevel, powMin, powMax))
       return VI_ERROR_PARAMETER3;
    if (rssmv_invalidViReal64Range (stopLevel, powMin, powMax))
       return VI_ERROR_PARAMETER4;
    if (rssmv_invalidViReal64Range (step, 0.0, 160.0))   
       return VI_ERROR_PARAMETER5;
    if (rssmv_invalidViReal64Range (dwellTime, 0.01, 5.0))   
       return VI_ERROR_PARAMETER6;

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:POW:MODE %s"
        ";STEP %Lf;DWEL %Lf;:SOUR:POW:STAR %Lf;STOP %Lf\n",
        SweepModeArr[mode], step, dwellTime, startLevel, stopLevel));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Level Sweep Mode.                                        */
/* Purpose:  This function specifies the sweep mode.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFLevelSweepMode (ViSession instrSession,
                                             ViInt32 mode)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (mode, 0, 2))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:POW:MODE %s\n", SweepModeArr[mode]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Start Level.                                             */
/* Purpose:  This function sets the staring value for a level sweep.         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFStartLevel (ViSession instrSession, ViReal64 level)
{                               
    ViStatus rssmv_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, ":SOUR:POW:STAR %Lf\n", level));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Stop Level.                                              */
/* Purpose:  This function sets the final value for a level sweep.           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFStopFreqLevel (ViSession instrSession,
                                            ViReal64 level)
{                               
    ViStatus rssmv_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, ":SOUR:POW:STOP %Lf\n", level));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Level Increment                                          */
/* Purpose:  This function defines the step width for the level setting.     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFLevelIncrement (ViSession instrSession,
                                            ViReal64 increment)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (increment, 0.1, 10.0))   
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:POW:STEP:INCR %Lf\n", increment));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Level Sweep Step.                                        */
/* Purpose:  This function sets the sweep step.                              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFLevelSweepStep (ViSession instrSession,
                                             ViReal64 step)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (step, 0.0, 160.0))   
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:POW:STEP %Lf\n", step));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Level Sweep Dweel Time.                                  */
/* Purpose:  This function sets the dwell time per level step.               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFLevelSweepDwellTime (ViSession instrSession,
                                                  ViReal64 time)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (time, 0.01, 5.0))    
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:POW:DWEL %Lf\n", time));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Level Sweep Manual.                                      */
/* Purpose:  This function sets the RF sweep level in Manual mode.           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFLevelSweepManual (ViSession instrSession,
                                               ViReal64 level)
{                               
    ViStatus rssmv_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, ":SOUR:POW:MAN %Lf\n", level));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Level Sweep Spacing                                      */
/* Purpose:  This function defines that the sweep steps have logarithmic     */
/*           spacings.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFLevelSweepSpacing (ViSession instrSession,
                                                ViInt32 spacing)
{
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (spacing, 1, 1))   
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:SWE:POW:SPAC %s\n", "LOG"));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Power Sweep Performed                                           */
/* Purpose:  This function queries whether a sweep is being performed.       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_powerSweepPerformed (ViSession instrSession, ViInt32 sweep,
                                  ViChar _VI_FAR sweepPerformed[])
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (sweep, 0, 2))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, "%s?\n", "%[^\r\n]", sweepPerformedArr[sweep],
        sweepPerformed));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Configure RF Level Correction                                   */
/* Purpose:  This function sets the correction state, list name, frequency   */
/*           and power points.
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_ConfigureRFLevelCorr (ViSession instrSession,
                                              ViBoolean state,
                                              ViString correctionListName,
                                              ViReal64 _VI_FAR frequencyPoints[],
                                              ViReal64 _VI_FAR powerPoints[],
                                              ViInt32 numberOfElements)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViBooleanRange (state))     
       return VI_ERROR_PARAMETER2;

    if (state) {
        if (rssmv_invalidViString (correctionListName, 7))   
           return VI_ERROR_PARAMETER3;
        if (!frequencyPoints)    
           return VI_ERROR_PARAMETER4;
        if (!powerPoints)    
           return VI_ERROR_PARAMETER5;

        if (numberOfElements > 0)   {
            CHECKERR (viPrintf (instrSession, ":SOUR:CORR:STAT ON;CSET:SEL \"%s\""
                ";DATA:FREQ %,*Lf;:SOUR:CORR:CSET:DATA:POW %,*Lf\n", correctionListName,
                numberOfElements, frequencyPoints,
                numberOfElements, powerPoints));
        }
        else {
            CHECKERR (viPrintf (instrSession, ":SOUR:CORR:STAT ON;CSET:SEL \"%s\"\n",
                correctionListName));
        }
    }
    else CHECKERR (viPrintf (instrSession, ":SOUR:CORR:STAT OFF\n"));

    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Level Correction State                                   */
/* Purpose:  This function sets correction table on or off.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFLevelCorrState (ViSession instrSession,
                                             ViBoolean state)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViBooleanRange (state))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:CORR:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Select / Create Correction List                                 */
/* Purpose:  This function select or creates correction list.                */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_selCorrList (ViSession instrSession,
                                     ViString correctionListName)
{                               
    ViStatus rssmv_status = VI_SUCCESS;

    if (rssmv_invalidViString (correctionListName, 7))   
       return VI_ERROR_PARAMETER2;
    
    CHECKERR (viPrintf (instrSession, ":SOUR:CORR:CSET:SEL \"%s\"\n",
            correctionListName));

    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Delete Correction List                                          */
/* Purpose:  This function deletes correction list.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_deleteCorrList (ViSession instrSession,
                                        ViString correctionListName)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViString (correctionListName, 7))   
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:CORR:CSET:DEL \"%s\"\n",
            correctionListName));
    
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Delete All Correction Lists                                     */
/* Purpose:  This function deletes all correction lists.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_deleteAllCorrLists (ViSession instrSession)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    CHECKERR (viPrintf (instrSession, ":SOUR:CORR:CSET:DEL:ALL\n"));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Get List Of Correction Lists                                    */
/* Purpose:  This function returns list of correction lists.                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_getListOfCorrLists (ViSession instrSession,
                                            ViChar _VI_FAR listOfCorrectionLists[])
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (!listOfCorrectionLists)  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":SOUR:CORR:CSET:CAT?\n", "%[^\r\n]",
        listOfCorrectionLists));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Frequency To Correction List                                */
/* Purpose:  This function sets the frequency array to correction list.      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setFreqToCorrList (ViSession instrSession,
                                           ViReal64 _VI_FAR frequency[],
                                           ViInt32 size)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (!frequency)
       return VI_ERROR_PARAMETER2;
    if ( size < 1 )
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":SOUR:CORR:CSET:DATA:FREQ %,*Lf\n",
        size, frequency));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Power To Correction List                                    */
/* Purpose:  This function sets the power array to correction list.          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setPowerToCorrList (ViSession instrSession,
                                            ViReal64 _VI_FAR power[],
                                            ViInt32 size)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (!power)
       return VI_ERROR_PARAMETER2;
    if ( size < 1 )
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":SOUR:CORR:CSET:DATA:POW %,*Lf\n",
        size, power));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Get Level Correction Points                                     */
/* Purpose:  This function returns the number of correction list elements.   */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_getLevelCorrPoints (ViSession instrSession,
                                            ViPInt32 points)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (!points)
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":SOUR:CORR:CSET:DATA:POW:POIN?\n", "%ld",
        points));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Get Level Frequency Points                                      */
/* Purpose:  This function returns the number of correction list elements.   */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_getFreqCorrPoints (ViSession instrSession,
                                            ViPInt32 points)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (!points)
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":SOUR:CORR:CSET:DATA:FREQ:POIN?\n", "%ld",
        points));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Get Free Space                                                  */
/* Purpose:  This function returns free space.                               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_getFreeCorrTables (ViSession instrSession,
                                            ViPInt32 freeSpace)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (!freeSpace)
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":SOUR:CORR:CSET:FREE?\n", "%ld",
        freeSpace));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Configure RF Output                                             */
/* Purpose:  This function sets the attenuation mode, attenuation and        */
/*           power on state.                                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_configRFOutput (ViSession instrSession,
                                        ViBoolean state,
                                        ViInt32 attenuationMode,
                                        ViReal64 reserved)
{
    ViStatus rssmv_status = VI_SUCCESS;

    if (rssmv_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    if (state) {
        if (rssmv_invalidViInt32Range (attenuationMode, 0, 1))
           return VI_ERROR_PARAMETER3;
    
        CHECKERR (viPrintf (instrSession, ":OUTP1:STAT ON;AMOD %s\n",
        RFAttModeArr[attenuationMode]));
    }
    else CHECKERR (viPrintf (instrSession, ":OUTP1:STAT OFF\n"));

    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Output State                                             */
/* Purpose:  This function sets RF output on or off.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFOutputState (ViSession instrSession,
                                          ViBoolean state)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP1:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Attenuation Mode                                         */
/* Purpose:  This function sets RF attenuation mode.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFAttMode (ViSession instrSession,
                                      ViInt32 attenuationMode)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (attenuationMode, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP1:AMOD %s\n", RFAttModeArr[attenuationMode]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Output Power On State                                    */
/* Purpose:  This function sets the state of RF output after reset.          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFOutPowOnState (ViSession instrSession,
                                            ViInt32 state)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (state,0,1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP1:PON %s\n", RFOutPonSArr[state]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Configure AM Modulation                                         */
/* Purpose:  This function sets the modulation depth, frequency and          */
/*           source.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_configAMModulation (ViSession instrSession, ViInt32 source,
                                            ViReal64 depth, ViReal64 frequency)
{
    ViStatus rssmv_status = VI_SUCCESS;

    if (rssmv_invalidViInt32Range (source, 0, 2))
       return VI_ERROR_PARAMETER2;
    if (rssmv_invalidViReal64Range (depth, 0.0, 100.0))
       return VI_ERROR_PARAMETER3;
    if (rssmv_invalidViReal64Range (frequency, 0.1, 1.0e6))
       return VI_ERROR_PARAMETER4;
    
    CHECKERR (viPrintf (instrSession, ":SOUR:AM:STAT ON;SOUR %s;DEPT %Lf"
        ";INT:FREQ %Lf\n", AMSourceArr[source], depth, frequency));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Configure FM Modulation                                         */
/* Purpose:  This function sets the modulation deviation, frequency and      */
/*           source.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_configFMModulation (ViSession instrSession, ViInt32 source,
                                            ViReal64 deviation, ViReal64 frequency)
{
    ViStatus rssmv_status = VI_SUCCESS;

    if (rssmv_invalidViInt32Range (source, 0, 2))
       return VI_ERROR_PARAMETER2;
    if (rssmv_invalidViReal64Range (deviation, 0.0, 1.0e6))
       return VI_ERROR_PARAMETER3;
    if (rssmv_invalidViReal64Range (frequency, 0.1, 1.0e6))
       return VI_ERROR_PARAMETER4;

    CHECKERR (viPrintf (instrSession, ":SOUR:FM:STAT ON;DEV %Lf;SOUR %s"
        ";INT:FREQ %Lf\n", deviation, AMSourceArr[source], frequency));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Configure PHM Modulation                                        */
/* Purpose:  This function sets the modulation deviation, frequency and      */
/*           source.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_configPHMModulation (ViSession instrSession, ViInt32 source,
                                            ViReal64 deviation, ViReal64 frequency)
{
    ViStatus rssmv_status = VI_SUCCESS;

    if (rssmv_invalidViInt32Range (source, 0, 2))
       return VI_ERROR_PARAMETER2;
    if (rssmv_invalidViReal64Range (deviation, 0.0, 10.0))
       return VI_ERROR_PARAMETER3;
    if (rssmv_invalidViReal64Range (frequency, 0.1, 1.0e6))
       return VI_ERROR_PARAMETER4;

    CHECKERR (viPrintf (instrSession, ":SOUR:PM:STAT ON;DEV %Lf;SOUR %s"
        ";INT:FREQ %Lf\n", deviation, AMSourceArr[source], frequency));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Configure PM Modulation                                         */
/* Purpose:  This function sets the modulation polarity and source           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_configPulseModulation (ViSession instrSession,
                                               ViInt32 source, ViInt32 polarity)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B3);

    if (rssmv_invalidViInt32Range (source, 0, 1))
       return VI_ERROR_PARAMETER2;
    if (rssmv_invalidViInt32Range (polarity, 0, 1))
       return VI_ERROR_PARAMETER3;
    
    CHECKERR (viPrintf (instrSession, ":SOUR:PULM:STAT ON;POL %s;SOUR %s\n",
        PolarityArr[polarity], PMSourceArr[source]));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Configure Stereo Modulation                                     */
/* Purpose:  This function configures Stereo modulation and switches it to on*/
/*           state.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_configStereoModulation (ViSession instrSession,
                                       ViReal64 MPXDeviation, ViInt32 source,
                                       ViInt32 stereoMode)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (rssmv_invalidViReal64Range (MPXDeviation, 0.0, 80.0e3))     
       return VI_ERROR_PARAMETER2;
    if (rssmv_invalidViInt32Range (source, 0, 2))    
       return VI_ERROR_PARAMETER3;
    if (rssmv_invalidViInt32Range (stereoMode, 0, 4))    
       return VI_ERROR_PARAMETER4;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER:STAT ON;DEV %Lf;SOUR %s;AUD:MODE %s\n",
        MPXDeviation, stereoSourceArr[source], stereoModeArr[stereoMode]));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Configure Vector Modulation                                     */
/* Purpose:  This function configures Vector modulation and switches it to on*/
/*           state.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_configVectorModulation (ViSession instrSession,
                                                ViBoolean impairmentState,
                                                ViReal64 leakage)
{
    ViStatus rssmv_status = VI_SUCCESS;

    if (rssmv_invalidViBooleanRange (impairmentState))
       return VI_ERROR_PARAMETER2;
    if (rssmv_invalidViReal64Range (leakage, 0.0, 50.0))
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":SOUR:DM:IQ ON;:DM:LEAK %Le;:DM:IMP %s\n",
        leakage, impairmentState ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set All Modulation State                                        */
/* Purpose:  This function deactivates all types of modulation with OFF. All */
/*           analog, vector, digital, digital standard and ARB modulations   */
/*           are thus set to OFF if they were switched on before.            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setAllModulationState (ViSession instrSession, ViBoolean state)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:MOD:ALL:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set AM State                                                    */
/* Purpose:  This function sets amplitude modulation on or off.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setAMState (ViSession instrSession, ViBoolean state)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:AM:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set AM Depth                                                    */
/* Purpose:  This function sets amplitude modulation depth.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setAMDepth (ViSession instrSession, ViReal64 depth)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (depth, 0.0, 100.0))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:AM:DEPT %Lf\n", depth));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set AM Frequency                                                */
/* Purpose:  This function sets the modulation frequency.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setAMFreq (ViSession instrSession, ViReal64 frequency)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (frequency, 0.1, 1.0e6))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:AM:INT:FREQ %Lf\n", frequency));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set AM Source                                                   */
/* Purpose:  This function selects the modulation source        .            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setAMSource (ViSession instrSession, ViInt32 source)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (source, 0, 2))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:AM:SOUR %s\n", AMSourceArr[source]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set AM External Coupling                                        */
/* Purpose:  This function selects the type of coupling for ext. source      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setAMExtCoupling (ViSession instrSession,
                ViInt32 input, ViInt32 coupling)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (input, 1, 1))
       return VI_ERROR_PARAMETER2;
    if (rssmv_invalidViInt32Range (coupling, 0, 1))
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":SOUR:AM:EXT%d:COUP %s\n", input,
        AMExtCoupArr[coupling]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set FM State                                                    */
/* Purpose:  This function sets frequency modulation on or off.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setFMState (ViSession instrSession, ViBoolean state)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
 
    if (rssmv_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:FM:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set FM Deviation.                                               */
/* Purpose:  This function sets the frequency variation.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setFMDeviation (ViSession instrSession,
                                        ViReal64 deviation)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (deviation, 0.0, 1.0e6))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:FM:DEV %Lf\n", deviation));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set FM Frequency                                                */
/* Purpose:  This function sets the frequency.                               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setFMFreq (ViSession instrSession, ViReal64 frequency)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (frequency, 0.1, 1.0e6))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:FM:INT:FREQ %Lf\n", frequency));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set FM Source                                                   */
/* Purpose:  This function selest the modulation source.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setFMSource (ViSession instrSession, ViInt32 source)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (source, 0, 2))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:FM:SOUR %s\n", AMSourceArr[source]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set FM External Coupling                                        */
/* Purpose:  This function selects the type of coupling for ext. source      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setFMExtCoup (ViSession instrSession, ViInt32 input,
                ViInt32 coupling)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (input, 1, 1))
       return VI_ERROR_PARAMETER2;
    if (rssmv_invalidViInt32Range (coupling, 0, 1))
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":SOUR:FM:EXT%d:COUP %s\n", input,
        AMExtCoupArr[coupling]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set FM Bandwidth                                                */
/* Purpose:  This function defines the fm bandwidth.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setFMBand (ViSession instrSession, ViInt32 band)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (band, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:FM:BAND %s\n", FMBandArr[band]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}
/*===========================================================================*/
/* Function: Set PHM State                                                   */
/* Purpose:  This function sets phase modulation on or off.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setPHMState (ViSession instrSession, ViBoolean state)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
 
    if (rssmv_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PM:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set PHM Deviation.                                              */
/* Purpose:  This function sets the phase variation.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setPHMDeviation (ViSession instrSession,
                                        ViReal64 deviation)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (deviation, 0.0, 10.0))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PM:DEV %Lf\n", deviation));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set PHM Frequency                                               */
/* Purpose:  This function sets the modulation frequency.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setPHMFreq (ViSession instrSession, ViReal64 frequency)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (frequency, 0.1, 1.0e6))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PM:INT:FREQ %Lf\n", frequency));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set PHM Source                                                  */
/* Purpose:  This function selects the modulation source.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setPHMSource (ViSession instrSession, ViInt32 source)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (source, 0, 2))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PM:SOUR %s\n", AMSourceArr[source]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set PHM External Coupling                                       */
/* Purpose:  This function selects the type of coupling for ext. source      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setPHMExtCoup (ViSession instrSession, ViInt32 coupling)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (coupling, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PM:EXT:COUP %s\n",
        AMExtCoupArr[coupling]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set PHM Bandwidth                                               */
/* Purpose:  This function defines the phm bandwidth.                        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setPHMBand (ViSession instrSession, ViInt32 band)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (band, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PM:BAND %s\n", FMBandArr[band]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set PM State                                                    */
/* Purpose:  This function sets pulse modulation on or off.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setPMState (ViSession instrSession, ViBoolean state)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B3);
 
    if (rssmv_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PULM:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set PM Polarity                                                 */
/* Purpose:  This function sets the polarity between modulating and          */
/*           modulated signal.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setPMPolar (ViSession instrSession, ViInt32 polarity)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B3);
    
    if (rssmv_invalidViInt32Range (polarity, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PULM:POL %s\n", PolarityArr[polarity]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set PM Source                                                   */
/* Purpose:  This function select the source of the modulating signal        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setPMSource (ViSession instrSession, ViInt32 source)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B3);

    if (rssmv_invalidViInt32Range (source, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PULM:SOUR %s\n", PMSourceArr[source]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: ALC Search                                                      */
/* Purpose:  This function returns ALC search results.                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_ALCSearch (ViSession instrSession,
                                   ViChar _VI_FAR searchResult[])
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (!searchResult)   
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":SOUR:POW:ALC:SEARCH?\n", "%s", searchResult));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Stereo State                                                */
/* Purpose:  This function is switching on/off stereo modulation.            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setStereoState (ViSession instrSession, ViBoolean modState)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (rssmv_invalidViBooleanRange (modState))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER:STAT %s\n", modState ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Stereo Deviation                                            */
/* Purpose:  This function sets frequency deviation of the stereo signal.    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setStereoDeviation (ViSession instrSession,
                                   ViReal64 deviation)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (rssmv_invalidViReal64Range (deviation, 0.0, 80.0e3))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER %Lf\n", deviation));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Stereo Source                                               */
/* Purpose:  This function selects the modulation sources for stereo         */
/*           modulation (the modulation sources cannot be used               */
/*           simultaneously).                                                */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setStereoSource (ViSession instrSession, ViInt32 source)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (rssmv_invalidViInt32Range (source, 0, 2))    
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER:SOUR %s\n", stereoSourceArr[source]));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Stereo Mode                                                 */
/* Purpose:  This function selects stereo modulation operating mode.         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setStereoMode (ViSession instrSession, ViInt32 stereoMode)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (rssmv_invalidViInt32Range (stereoMode, 0, 4))    
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER:MODE %s\n", stereoModeArr[stereoMode]));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Stereo LF Gen Frequency                                     */
/* Purpose:  This function sets input value of the frequency of the LF       */
/*           generator.                                                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setStereoLFGenFrequency (ViSession instrSession,
                                        ViReal64 LFGenFrequency)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (rssmv_invalidViReal64Range (LFGenFrequency, 0.1, 1.0e6))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER:AUD %Lf\n", LFGenFrequency));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Stereo Ext Impedance                                        */
/* Purpose:  This function sets the input impedances of the analog audio     */
/*           inputs L and R. Both input impedances are switched              */
/*           simultaneously.                                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setStereoExtImpedance (ViSession instrSession,
                                      ViInt32 impedance)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (rssmv_invalidViInt32Range (impedance, 0, 1))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER:EXT:IMP %s\n",
        stereoImpedanceArr[impedance]));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Stereo Preemphasis                                          */
/* Purpose:  This function selects the stereo modulation preemphasis.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setStereoPreemphasis (ViSession instrSession,
                                     ViInt32 preemphasis)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (rssmv_invalidViInt32Range (preemphasis, 0, 2))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER:AUD:PRE%s\n",
        pinstrSession[preemphasis]));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Stereo Pilot State                                          */
/* Purpose:  This function is switching on/off the pilot tone.               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setStereoPilotState (ViSession instrSession,
                                    ViBoolean pilotState)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (rssmv_invalidViBooleanRange (pilotState))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER:PIL:STAT %s\n", pilotState ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Stereo Pilot Deviation                                      */
/* Purpose:  This function sets value of the frequency deviation of the pilot*/
/*           tone.                                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setStereoPilotDeviation (ViSession instrSession,
                                        ViReal64 pilotDeviation)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (rssmv_invalidViReal64Range (pilotDeviation, 0.0, 10.0e3))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER:PIL %Lf\n", pilotDeviation));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Stereo Pilot Phase                                          */
/* Purpose:  This function sets value of phase of the pilot tone.            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setStereoPilotPhase (ViSession instrSession,
                                    ViReal64 pilotPhase)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (rssmv_invalidViReal64Range (pilotPhase, -5.0, 5.0))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER:PIL:PHA %Le\n", pilotPhase));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Stereo ARI State                                            */
/* Purpose:  This function is switching on/off ARI area code and braodcasting*/
/*           code.                                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setStereoARIState (ViSession instrSession, ViBoolean ARIState)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (rssmv_invalidViBooleanRange (ARIState))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER:ARI:STAT %s\n", ARIState ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Stereo ARI Deviation                                        */
/* Purpose:  This function inputs value of the frequency deviation of the ARI*/
/*           subcarrier.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setStereoARIDeviation (ViSession instrSession,
                                      ViReal64 ARIDeviation)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (rssmv_invalidViReal64Range (ARIDeviation, 0.0, 10.0e3))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER:ARI %Lf\n", ARIDeviation));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Stereo ARI Identification                                   */
/* Purpose:  This function selects between ARI broadcasting code (DK) and    */
/*           traffic area code (BK).                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setStereoARIIdentification (ViSession instrSession,
                                           ViInt32 ARIIdentification)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (rssmv_invalidViInt32Range (ARIIdentification, 0, 3))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER:TYPE%s\n",
        ARIIdentificationArr[ARIIdentification]));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Stereo ARI BK Code                                          */
/* Purpose:  This function selects the standard traffic area codes.          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setStereoARIBKCode (ViSession instrSession, ViInt32 BKCode)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (rssmv_invalidViInt32Range (BKCode, 0, 5))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER:ARI:BK %s\n", BKCodeArr[BKCode]));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Stereo RDS State                                            */
/* Purpose:  This function switches on/off RDS function.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setStereoRDSState (ViSession instrSession, ViBoolean RDSState)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (rssmv_invalidViBooleanRange (RDSState))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER:RDS:STAT %s\n", RDSState ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Stereo RDS Deviation                                        */
/* Purpose:  This function sets value of the frequency deviation of the RDS  */
/*           subcarrier.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setStereoRDSDeviation (ViSession instrSession,
                                      ViReal64 RDSDeviation)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (rssmv_invalidViReal64Range (RDSDeviation, 0.0, 10.0e3))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER:RDS %Lf\n", RDSDeviation));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Stereo RDS Data Set                                         */
/* Purpose:  This function selects and activation of the RDS data set.       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setStereoRDSDataSet (ViSession instrSession, ViInt32 RDSDataSet)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);


    if (rssmv_invalidViInt32Range (RDSDataSet, 0, 4))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER:RDS:DAT %s\n",
        RDSDataSetArr[RDSDataSet]));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Stereo RDS Traffic Program                                  */
/* Purpose:  This function is switching on/off traffic programm.             */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setStereoRDSTrafficProgram (ViSession instrSession,
                                           ViBoolean RDSTrafficProgramState)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (rssmv_invalidViBooleanRange (RDSTrafficProgramState))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER:RDS:TRAF:PROG:STAT %s\n",
        RDSTrafficProgramState ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Stereo RDS Traffic Announcement                             */
/* Purpose:  This function is switching on/off traffic announcement.         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setStereoRDSTrafficAnnouncement (ViSession instrSession,
                ViBoolean RDSTrafficAnnouncementState)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (rssmv_invalidViBooleanRange (RDSTrafficAnnouncementState))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:STER:RDS:TRAF:ANN %s\n",
        RDSTrafficAnnouncementState ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Write RDS Command                                               */
/* Purpose:  This function sends the RDS setting (query) strings to the      */
/*           RDS/Stereo coder.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_writeRDSCommand (ViSession instrSession, ViString RDSCommand)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (RDSCommand)
        {
        CHECKERR (viPrintf (instrSession, ":SOUR:STER:DIR \"%s\"\n", RDSCommand));
        CHECKERR (rssmv_checkStatus (instrSession));
        }

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Read RDS Response                                               */
/* Purpose:  This function sends the RDS query string to the RDS/Stereo coder*/
/*           and immediately gets the response.                              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_readRDSResponse (ViSession instrSession, ViString RDSQuery,
                                ViInt32 numberBytesToRead, ViPInt32 numBytesRead,
                                ViChar _VI_FAR readBuffer[])
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;
    ViUInt32            tmpCnt = 0;
    ViChar              *tmpBuf;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B5);

    if (RDSQuery)
        CHECKERR (viPrintf (instrSession, ":SOUR:STER:DIR? \"%s\"\n", RDSQuery));

    if (numBytesRead)
        *numBytesRead = 0L;

    if (readBuffer)
        {

        /* Allocate temporary buffer which allows to skip '"', '\r', '\n' ... termination characters */
        if (!(tmpBuf = malloc (numberBytesToRead + 4)))
            return VI_ERROR_ALLOC;

        if ((rssmv_status = viRead (instrSession, tmpBuf, numberBytesToRead + 3, &tmpCnt)) < 0)
            {
            if (tmpBuf)
                free (tmpBuf);
            return rssmv_status;
            }

        tmpBuf[(numberBytesToRead < tmpCnt) ? numberBytesToRead : tmpCnt] = 0;
        sscanf (tmpBuf + 1, "%[^\"\r\n]", readBuffer);

        if (tmpBuf)
            free (tmpBuf);
        if (numBytesRead)
            *numBytesRead = strlen (readBuffer);

        CHECKERR (rssmv_checkStatus (instrSession));
        }

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Vector State                                                */
/* Purpose:  This function is switching on/off vector modulation.            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setVectorState (ViSession instrSession,
                                        ViBoolean modState)
{
    ViStatus rssmv_status = VI_SUCCESS;

    if (rssmv_invalidViBooleanRange (modState))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:DM:IQ %s\n", modState ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Vector Impairment State                                     */
/* Purpose:  This function is switching I/Q impairment on and off.           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setVectorImpairmentState (ViSession instrSession,
                                                  ViBoolean impairmentState)
{
    ViStatus rssmv_status = VI_SUCCESS;

    if (rssmv_invalidViBooleanRange (impairmentState))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:DM:IMP %s\n", impairmentState ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Vector Leakage                                              */
/* Purpose:  This function adjusts the residual carrier amplitude for vector */
/*           modulation.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setVectorLeakage (ViSession instrSession,
                                          ViReal64 leakage)
{
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (leakage, 0.0, 50.0))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:DM:LEAK:MAGN %Le\n", leakage));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Vector Imbalance                                            */
/* Purpose:  This function adjusts the ratio of I and Q modulation (gain     */
/*           balance).                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setVectorImbalance (ViSession instrSession,
                                            ViReal64 imbalance)
{
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (imbalance, -12.0, 12.0))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:DM:IQR:MAGN %Le\n", imbalance));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Vector Quadrature Offset                                    */
/* Purpose:  This function changes the quadrature offset for vector          */
/*           modulation.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setVectorQuadratureOffset (ViSession instrSession,
                                                   ViReal64 quadratureOffset)
{
    ViStatus rssmv_status = VI_SUCCESS;

    if (rssmv_invalidViReal64Range (quadratureOffset, -10.0, 10.0))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:DM:QUAD:ANGL %Le\n", quadratureOffset));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Vector IQ Swap                                              */
/* Purpose:  This function selects between normal and inverted I/Q           */
/*           modulation. Interchanging the I and Q signals inverts the       */
/*           modulation sidebands.                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setVectorIQSwap (ViSession instrSession,
                                         ViBoolean IQSwap)
{
    ViStatus rssmv_status = VI_SUCCESS;

    if (rssmv_invalidViBooleanRange (IQSwap))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:DM:IQSW %s\n", IQSwap ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set ALC State                                                   */
/* Purpose:  This function sets the ALC on or off.                           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setALCState (ViSession instrSession, ViBoolean state)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
 
    if (rssmv_invalidViBooleanRange (state))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:POW:ALC:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Configure LF Output                                             */
/* Purpose:  This function sets the output voltage, frequency, function and  */
/*           mode.                                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_configLFOutput (ViSession instrSession, ViBoolean state,
                                        ViReal64 frequency, ViReal64 voltage)
{
    ViStatus rssmv_status = VI_SUCCESS;

    if (rssmv_invalidViBooleanRange (state))     
       return VI_ERROR_PARAMETER2;

    if (state) {
        if (rssmv_invalidViReal64Range (frequency, 0.1, 1.0e6))
           return VI_ERROR_PARAMETER3;
        if (rssmv_invalidViReal64Range (voltage, 0.0, 4.0))
           return VI_ERROR_PARAMETER4;

        CHECKERR (viPrintf (instrSession, ":SOUR2:FREQ:MODE CW;:OUTP2:STAT ON;"
            ":OUTP2:VOLT %LfV;:SOUR2:FREQ:CW %Lf\n",
            voltage, frequency));
    }
    else CHECKERR (viPrintf (instrSession, ":OUTP2:STAT OFF\n"));

    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set LF Voltage                                                  */
/* Purpose:  This function sets the ALC on or off.                           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setLFVoltage (ViSession instrSession, ViReal64 voltage)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (voltage, 0.0, 4.0))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP2:VOLT %LfV\n", voltage));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set LF Output State                                             */
/* Purpose:  This function sets the LF output on or off.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setLFOutputState (ViSession instrSession,
                                          ViBoolean state)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
 
    if (rssmv_invalidViBooleanRange (state))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP2:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set LF CW Frequency                                             */
/* Purpose:  This function sets the LF frequency.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setLFCWFreq (ViSession instrSession, ViReal64 frequency)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (frequency, 0.1, 1.0e6))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR2:FREQ:CW %Lf\n", frequency));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set LF Frequency Mode                                           */
/* Purpose:  This function selects the operating mode of LF subsystem.       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setLFFreqMode (ViSession instrSession, ViInt32 mode)
{                               
    ViStatus rssmv_status = VI_SUCCESS;

    if (rssmv_invalidViInt32Range (mode, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR2:FREQ:MODE %s\n", RFFreqModeArr[mode]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Configure LF Sweep                                              */
/* Purpose:  This function sets the sweep mode, spacing, dwell time and step.*/
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_configLFSweep (ViSession instrSession, ViInt32 mode,
                                       ViReal64 startFrequency,
                                       ViReal64 stopFrequency, ViReal64 step,
                                       ViReal64 dwellTime)
{
    ViStatus rssmv_status = VI_SUCCESS;

    if (rssmv_invalidViInt32Range (mode, 0, 2))
       return VI_ERROR_PARAMETER2;
    if (rssmv_invalidViReal64Range (startFrequency, 0.1, 1.0e6))     
       return VI_ERROR_PARAMETER3;
    if (rssmv_invalidViReal64Range (stopFrequency, 0.1, 1.0e6))  
       return VI_ERROR_PARAMETER4;
    if (rssmv_invalidViReal64Range (step, 0.0, 1.0e6))   
       return VI_ERROR_PARAMETER5;
    if (rssmv_invalidViReal64Range (dwellTime, 0.01, 5.0))   
       return VI_ERROR_PARAMETER6;

    CHECKERR (viPrintf (instrSession, ":SOUR2:FREQ:MODE SWE;:SOUR2:SWE:FREQ:MODE %s;"
        ":SOUR2:FREQ:STAR %Lf;STOP %Lf;:SOUR2:SWE:FREQ:DWELL %Lf;STEP:LIN %Lf\n",
        SweepModeArr[mode], startFrequency, stopFrequency, dwellTime, step));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set LF Sweep Mode                                               */
/* Purpose:  This function specifies the run of the sweep.                   */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setLFSweepMode (ViSession instrSession, ViInt32 mode)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (mode, 0, 2))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR2:SWE:FREQ:MODE %s\n", SweepModeArr[mode]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set LF CW Start Frequency                                       */
/* Purpose:  This function sets the LF start frequency.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setLFStartFreq (ViSession instrSession,
                                        ViReal64 frequency)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (frequency, 0.1, 1.0e6))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR2:FREQ:STAR %Lf\n", frequency));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set LF CW Stop Frequency                                        */
/* Purpose:  This function sets the LF stop frequency.                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setLFStopFreq (ViSession instrSession,
                                       ViReal64 frequency)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (frequency, 0.1, 1.0e6))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR2:FREQ:STOP %Lf\n", frequency));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set LF CW Frequency Manual                                      */
/* Purpose:  This function sets the LF frequency in manual mode.             */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setLFFreqMan (ViSession instrSession,
                                      ViReal64 frequency)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (frequency, 0.1, 1.0e6))  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR2:FREQ:MAN %Lf\n", frequency));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set LF Sweep Spacing                                            */
/* Purpose:  This function specifies the steps have linear or logarithmic    */
/*           spacing                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setLFSweepSpac (ViSession instrSession, ViInt32 spacing)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (spacing, 0, 1))   
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR2:SWE:FREQ:SPAC %s\n",
        LFFreqSweepSpacArr[spacing]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set LF Sweep Step                                               */
/* Purpose:  This function specifies the step width in linear or logarithmic */
/*           mode.                                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setLFSweepStep (ViSession instrSession, ViInt32 mode,
                                        ViReal64 step)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (mode, 0, 1))  
       return VI_ERROR_PARAMETER2;

    if (mode == 0) {
        if (rssmv_invalidViReal64Range (step, 0.0, 1.0e6))   
           return VI_ERROR_PARAMETER3;
    }           
    else {
        if (rssmv_invalidViReal64Range (step, 0.01, 100.0))  
           return VI_ERROR_PARAMETER3;
    }           
    
    CHECKERR (viPrintf (instrSession, ":SOUR2:SWE:FREQ:STEP:%s %Lf\n",
        LFFreqSweepSpacArr[mode], step));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set LF Sweep Dweel Time.                                        */
/* Purpose:  This function sets the dwell time per frequency step.           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setLFSweepDwellTime (ViSession instrSession,
                                             ViReal64 time)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (time, 0.01, 5.0))    
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR2:SWE:FREQ:DWEL %Lf\n", time));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Configure Pulse Output.                                         */
/* Purpose:  This function sets the pulse source, polarity, pulse period,    */
/*           width, delay and V/GHz scale                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_configPulseOutput (ViSession instrSession,
                                           ViInt32 polarity, ViReal64 pulsePeriod,
                                           ViReal64 pulseWidth, ViReal64 pulseDelay)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B3);

    if (rssmv_invalidViInt32Range (polarity, 0, 1))
       return VI_ERROR_PARAMETER2;
    if (rssmv_invalidViReal64Range (pulsePeriod, 100.0e-9, 85.0))
       return VI_ERROR_PARAMETER3;
    if (rssmv_invalidViReal64Range (pulseWidth, 20.0e-9, 1.3))
       return VI_ERROR_PARAMETER4;
    if (rssmv_invalidViReal64Range (pulseDelay, 20.0e-9, 1.3))
       return VI_ERROR_PARAMETER5;

    CHECKERR (viPrintf (instrSession, ":OUTP3:SOUR PULS;POL:PULS %s"
        ";:SOUR:PULS:PER %Le;WIDT %Le;DEL %Le\n", PolarityArr[polarity],
        pulsePeriod, pulseWidth, pulseDelay));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Configure Video Output.                                         */
/* Purpose:  This function sets the video scale and sets video output on.    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_configVideoOutput (ViSession instrSession,
                                           ViInt32 reserved)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B3);

    CHECKERR (viPrintf (instrSession, ":OUTP3:SOUR VID\n"));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Pulse Source                                                */
/* Purpose:  This function selects the pulse generator or video output.      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setPulseSource (ViSession instrSession, ViInt32 source)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B3);
    
    if (rssmv_invalidViInt32Range (source, 0, 2))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP3:SOUR %s\n", pulseOutSourceArr[source]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Pulse Polarity                                              */
/* Purpose:  This function selects the polarity of the signal.               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setPulsePolarity (ViSession instrSession,
                                          ViInt32 polarity)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B3);
    
    if (rssmv_invalidViInt32Range (polarity, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP3:POL:PULS %s\n", PolarityArr[polarity]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set V/GHz Output Scale                                          */
/* Purpose:  This function defines the signal scale at V/GHz output.         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setVGHzOutputScale (ViSession instrSession,
                                            ViInt32 SignalScale)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (SignalScale, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":OUTP3:SCAL %s\n", SignalScale ? "1" : "0.5"));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Pulse Period                                                */
/* Purpose:  This function sets the pulse period.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setPulsePeriod (ViSession instrSession, ViReal64 period)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B3);
    
    if (rssmv_invalidViReal64Range (period, 100.0e-9, 85.0))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PULS:PER %Le\n", period));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Pulse Width                                                 */
/* Purpose:  This function sets the pulse width.                             */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setPulseWidth (ViSession instrSession, ViReal64 width)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B3);
    
    if (rssmv_invalidViReal64Range (width, 20.0e-9, 1.3))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PULS:WIDT %Le\n", width));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Pulse Delay                                                 */
/* Purpose:  This function sets the pulse delay.                             */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setPulseDelay (ViSession instrSession, ViReal64 delay)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B3);
    
    if (rssmv_invalidViReal64Range (delay, 20.0e-9, 1.3))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PULS:DEL %Le\n", delay));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Double Pulse State                                          */
/* Purpose:  This function switches the second pulse on or off.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setDoublePulseState (ViSession instrSession,
                                             ViBoolean state)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B3);
 
    if (rssmv_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PULS:DOUB:STAT %s\n",
        state ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Double Pulse Delay                                          */
/* Purpose:  This function sets the double pulse delay.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setDoublePulseDelay (ViSession instrSession,
                                             ViReal64 delay)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKOPT (RSSMV_OPT_B3);
    
    if (rssmv_invalidViReal64Range (delay, 60.0e-9, 1.3))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PULS:DOUB:DEL %Le\n", delay));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*============================================================================*/
/* Function: Configure Ref Oscillator State                                   */
/* Purpose:  This function sets ref. oscilator state, source and adjust value */
/*============================================================================*/
ViStatus _VI_FUNC rssmv_configureRefOscilator (ViSession instrSession,
                                               ViBoolean state, ViInt32 source,
                                               ViInt32 adjustValue)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
 
    if (rssmv_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    if (state) {
        if (rssmv_invalidViInt32Range (source, 0, 1))
            return VI_ERROR_PARAMETER3;
        if (rssmv_invalidViInt32Range (adjustValue, -2048, 2047))
            return VI_ERROR_PARAMETER4;
        CHECKERR (viPrintf (instrSession, ":SOUR:ROSC:INT:ADJ:STAT ON;VAL %ld"
            ";:SOUR:ROSC:SOUR %s\n", adjustValue, RFOscSourceArr[source]));
    }
    else CHECKERR (viPrintf (instrSession, ":SOUR:ROSC:INT:ADJ:STAT OFF\n"));

    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Oscillator State                                         */
/* Purpose:  This function switches the  frequency adjustment on or off.     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRFOscState (ViSession instrSession, ViBoolean state)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
 
    if (rssmv_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:ROSC:INT:ADJ:STAT %s\n",
        state ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Oscillator Source                                        */
/* Purpose:  This function selects the reference source.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRefOscSource (ViSession instrSession, ViInt32 source)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (source, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:ROSC:SOUR %s\n", RFOscSourceArr[source]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Oscillator Adjust Value                                  */
/* Purpose:  This function sets the frequency correction value.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRefOscAdjVal (ViSession instrSession, ViInt32 value)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (value, 0, 4095))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:ROSC:INT:ADJ:VAL %ld\n", value));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set RF Oscillator Loop                                          */
/* Purpose:  This function selects the reference loop.                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRefOscRLoop (ViSession instrSession, ViInt32 loop)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (loop, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:ROSC:RLO %s\n", RFOscRLoopArr[loop]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Ref Phase State                                             */
/* Purpose:  This function switches reference phase setting on or off.       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRefPhaseState (ViSession instrSession,
                                          ViBoolean refPhaseState)
{
    ViStatus rssmv_status = VI_SUCCESS;
 
    if (rssmv_invalidViBooleanRange (refPhaseState))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PHAS:STAT %s\n",
        refPhaseState ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Ref Phase                                                   */
/* Purpose:  This function sets value of the phase of the output signal with */
/*           respect to a reference signal of the same frequency.            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setRefPhase (ViSession instrSession,
                                     ViReal64 phaseValue)
{
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViReal64Range (phaseValue, -360.0, 360.0))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SOUR:PHAS %Lf\n", phaseValue));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Zero Ref Phase                                                  */
/* Purpose:  This function sets the phase value to 0. The phase of output    */
/*           signal will not be changed.                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_zeroRefPhase (ViSession instrSession)
{
    ViStatus rssmv_status = VI_SUCCESS;
    
    CHECKERR (viPrintf (instrSession, ":SOUR:PHAS:REF\n"));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Sweep Trigger Source                                        */
/* Purpose:  This function selects sweep trigger source.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setSweepTrigSource (ViSession instrSession,
                                            ViInt32 generator, ViInt32 source)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (generator, 1, 2))
       return VI_ERROR_PARAMETER2;
    if (rssmv_invalidViInt32Range (source, 0, 2))
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":TRIG%ld:SWE:SOUR %s\n", generator,
        TriggerSourceArr[source]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Pulse Trigger Source                                        */
/* Purpose:  This function selects pulse trigger source.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setPulseTrigSource (ViSession instrSession,
                                            ViInt32 source)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (source, 0, 2))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":TRIG:PULS:SOUR %s\n",
        PulseTriggerSourceArr[source]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Pulse Trigger Slope                                         */
/* Purpose:  This function selects pulse trigger slope.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setPulseTrigSlope (ViSession instrSession,
                                           ViInt32 slope)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (slope, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":TRIG:PULS:SLOP %s\n",
        PulseTriggerSlopeArr[slope]));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Pulse Trigger Polarity                                      */
/* Purpose:  This function defines the active level of the gate signal       */
/*           (Normal - Active level = HIGH, Inverted Active level = LOW).    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setPulseTrigPolarity (ViSession instrSession,
                                     ViInt32 triggerPolarity)
{
    ViStatus rssmv_status = VI_SUCCESS;

    if (rssmv_invalidViInt32Range (triggerPolarity, 0, 1))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":TRIG:PULS:EGAT:POL %s\n",
        triggerPolarityArr[triggerPolarity]));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Level Calibration State                                     */
/* Purpose:  This function sets the level calibration on or off.             */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setLevCalState (ViSession instrSession,
                                          ViBoolean state)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
 
    if (rssmv_invalidViBooleanRange (state))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":CAL:LEV:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Att Correction State                                        */
/* Purpose:  This function sets the attenuation correction on or off.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_setAttCorrState (ViSession instrSession,
                                          ViBoolean state)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
 
    if (rssmv_invalidViBooleanRange (state))     
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":CAL:ATT:STAT %s\n", state ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Get Ref Osc Calib Value                                         */
/* Purpose:  This function gets calibration value for reference oscillator.  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_getRefOscCalVal (ViSession instrSession,
                                         ViPInt32 value)
{
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (!value)  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":CAL:ROSC?\n", "%ld", value));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Store Ref Osc Calib Value                                       */
/* Purpose:  This function stores calibration value for reference oscillator.*/
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_storeRefOscCalVal (ViSession instrSession)
{
    ViStatus rssmv_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, ":CAL:ROSC:STOR\n"));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Send Trigger                                                    */
/* Purpose:  This function sends selected trigger.                           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_SendTrigger (ViSession instrSession, ViInt32 generator,
                                     ViInt32 systemToTrigger)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (generator, 1, 2))
       return VI_ERROR_PARAMETER2;
    if (rssmv_invalidViInt32Range (systemToTrigger, 0, 0))
       return VI_ERROR_PARAMETER3;
    
    if (systemToTrigger == 0) {
        CHECKERR (viPrintf (instrSession, ":TRIG%d:%s:IMM\n", generator,
            TriggerArr[systemToTrigger]));  
    }
    else {
        CHECKERR (viPrintf (instrSession, ":TRIG:%s:IMM\n",
            TriggerArr[systemToTrigger]));
    }
    
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Abort Trigger                                                   */
/* Purpose:  This function aborts selected trigger.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_abortTrigger (ViSession instrSession,
                                      ViInt32 systemToAbort)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (systemToAbort, 0, 0))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":ABOR:%s\n", TriggerArr[systemToAbort]));      
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Calibration                                                     */
/* Purpose:  This function calibrates selected component.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_utilCalibration (ViSession instrSession, ViInt32 toCal,
                ViInt32 timeout, ViChar _VI_FAR CalResults[])
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    ViInt32 oldTimeout;

    if (rssmv_invalidViInt32Range (toCal, 0, 9))
        return VI_ERROR_PARAMETER2;
    if (rssmv_invalidViInt32Range (timeout, 0, 1800000))
        return VI_ERROR_PARAMETER3;
    if (!CalResults)
        return VI_ERROR_PARAMETER4;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &oldTimeout));
    CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, timeout));
    CHECKERR (viQueryf (instrSession, "%s?\n", "%[^\r\n]", CalArr[toCal], CalResults));
    CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, oldTimeout));

    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: System Keylock                                                  */
/* Purpose:  This function sets the local lockout.                           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_utilSystemKeylock (ViSession instrSession,
                ViBoolean keylock)
{
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViBooleanRange (keylock))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SYST:KLOC %s\n", keylock ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Recall Setup                                                    */
/* Purpose:  This function recalls setup from memory.                        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_utilRecallSetup (ViSession instrSession,
                                            ViInt32 registerNo)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (registerNo, 1, 100))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, "*RCL %Ld\n", registerNo));     
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Save Setup                                                      */
/* Purpose:  This function saves setup to memory.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_utilSaveSetup (ViSession instrSession,
                                          ViInt32 registerNo)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (registerNo, 1, 100))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, "*SAV %Ld\n", registerNo));     
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Get Free Settings                                               */
/* Purpose:  This function returns the number of free memories.              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_utilGetFreeSett (ViSession instrSession, ViPInt32 value)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (!value)  
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":MEM:NST?\n", "%ld", value));
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Register                                                    */
/* Purpose:  This function sets value to  selected register.                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_utilSetReg (ViSession instrSession,
                                    ViInt32 registerName, ViInt32 value)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (registerName, 0, 13))     
       return VI_ERROR_PARAMETER2;
    if (registerName > 9) { 
        if (rssmv_invalidViInt32Range (value, 0, 255))   
            return VI_ERROR_PARAMETER3;
    }
    else {
        if (rssmv_invalidViInt32Range (value, 0, 32767))     
            return VI_ERROR_PARAMETER3;
    }

    if ((registerName!=0) && (registerName!=1) && (registerName!=5) &&
        (registerName!=6) && (registerName!=11) && (registerName!=13))
        {
        CHECKERR (viPrintf (instrSession, "%s %ld\n", RegisterArr[registerName],
            value));
        }

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Get Register                                                    */
/* Purpose:  This function returns value from  selected register.            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_utilGetReg (ViSession instrSession,
                                    ViInt32 registerName, ViPInt32 value)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (registerName, 0, 13))
       return VI_ERROR_PARAMETER2;
    if (!value)
       return VI_ERROR_PARAMETER3;

    CHECKERR (viQueryf (instrSession, "%s?\n", "%ld", RegisterArr[registerName],
        value));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Display Mode                                                */
/* Purpose:  This function selects mode of the specified display.            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_utilSetDisplayMode (ViSession instrSession,
                                            ViInt32 parameter, ViBoolean state)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (parameter, 0, 2))
       return VI_ERROR_PARAMETER2;
    if (rssmv_invalidViBooleanRange (state))
       return VI_ERROR_PARAMETER3;

    CHECKERR (viPrintf (instrSession, ":DISP:ANN:%s %s\n", DisplayParamArr[parameter],
        state ? "ON" : "OFF"));     
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set Display Update                                              */
/* Purpose:  This function switches on or off the update of all instruments  */
/*           display elements.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_utilSetDisplayUpdate (ViSession instrSession,
                ViBoolean displayUpdate)
{
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViBooleanRange (displayUpdate))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SYST:DISP:UPD %s\n",
        displayUpdate ? "ON" : "OFF"));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Set GPIB Address                                                */
/* Purpose:  This function sets GPIB addres of the instrument.               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_utilSetGPIBAddr (ViSession instrSession,
                                         ViInt32 address)
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (address, 1, 30))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viPrintf (instrSession, ":SYST:COMM:GPIB:SELF:ADDR %ld\n", address));       
    CHECKERR (rssmv_checkStatus (instrSession));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Configure RS232.                                                */
/* Purpose:  This function sets the baud rate, data bits, stop bits, hardware*/
/*           and software hadshake and parity                                */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_utilConfigureRS232 (ViSession instrSession,
                ViInt32 baudRate, ViInt32 dataBits, ViInt32 stopBits,
                ViInt32 hardwareHandshake, ViInt32 softwareHandshake,
                ViInt32 parity)
{
    ViStatus rssmv_status = VI_SUCCESS;

    if (rssmv_invalidViInt32Range (baudRate,0,7))
       return VI_ERROR_PARAMETER2;
    if (rssmv_invalidViInt32Range (dataBits, 7, 8))
       return VI_ERROR_PARAMETER3;
    if (rssmv_invalidViInt32Range (stopBits, 1, 2))
       return VI_ERROR_PARAMETER4;
    if (rssmv_invalidViInt32Range (hardwareHandshake, 0, 3))
       return VI_ERROR_PARAMETER5;
    if (rssmv_invalidViInt32Range (softwareHandshake, 0, 1))
       return VI_ERROR_PARAMETER6;
    if (rssmv_invalidViInt32Range (parity, 0, 2))
       return VI_ERROR_PARAMETER7;

    if (hardwareHandshake == 3) /* Hardware handshake does not apply. */
        CHECKERR (viPrintf (instrSession, ":SYST:COMM:SER:BAUD %s;BITS %ld"
            ";SBIT %ld;PACE %s;PAR %s\n", BaudRateArr[baudRate],
            dataBits, stopBits, softHandArr[softwareHandshake], ParityArr[parity]));
    else
        CHECKERR (viPrintf (instrSession, ":SYST:COMM:SER:BAUD %s;BITS %ld"
            ";SBIT %ld;PACE %s;PAR %s;CONT:RTS %s\n", BaudRateArr[baudRate],
            dataBits, stopBits, softHandArr[softwareHandshake], ParityArr[parity],
            hardHandArr[hardwareHandshake]));

    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Test                                                            */
/* Purpose:  This function tests selected component.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_utilTest (ViSession instrSession, ViInt32 toTest,
                                  ViChar _VI_FAR testResults[])
{                               
    ViStatus rssmv_status = VI_SUCCESS;
    
    if (rssmv_invalidViInt32Range (toTest, 0, 2))
       return VI_ERROR_PARAMETER2;

    CHECKERR (viQueryf (instrSession, ":TEST:%s?\n", "%[^\r\n]", TestArr[toTest],
        testResults));
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Flush Error Queue                                               */
/* Purpose:  This function deletes all error queue.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_FlushErrorQueue (ViSession instrSession)
{
    ViStatus rssmv_status = VI_SUCCESS;
    ViInt32 errorCode = 0;

    do {
        CHECKERR (viPrintf (instrSession, ":STAT:QUE:NEXT?\n"));
        CHECKERR (viScanf (instrSession, "%ld,%*[^\r\n]", &errorCode));
    }
    while (errorCode != 0);

    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/

/*****************************************************************************/
/*-------- INSERT USER-CALLABLE INSTRUMENT-DEPENDENT ROUTINES HERE ----------*/
/*****************************************************************************/
           
/*===========================================================================*/
/* Function: State Checking                                                  */
/* Purpose:  This function switches state checking of the instrument (reading*/
/*           of the Standard Event Register and checking it for error) status*/
/*           subsystem. Driver functions are using state checking which is by*/
/*           default enabled.                                                */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_errorCheckState (ViSession instrSession,
                                          ViBoolean state)
{
    ViStatus rssmv_status = VI_SUCCESS;
    rssmv_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    instrPtr -> errorChecking = state;

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_writeInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViStatus rssmv_status = VI_SUCCESS;
    
    CHECKERR (viPrintf (instrSession, "%s\n", writeBuffer));
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Read Instrument Buffer                                          */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_readInstrData (ViSession instrSession,
                                        ViInt32 numberBytesToRead,
                                        ViChar _VI_FAR readBuffer[],
                                        ViPInt32 numBytesRead)
{
    ViStatus rssmv_status = VI_SUCCESS;
    ViUInt32 tmpCnt;

    if (numBytesRead)
        *numBytesRead = 0L;
        
    CHECKERR (viRead (instrSession, (ViBuf)readBuffer, numberBytesToRead, &tmpCnt));
    
    if (numBytesRead)
        *numBytesRead = (ViInt32)tmpCnt;
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.  If the reset function     */
/*           is not supported by the instrument, this function returns       */
/*           the warning VI_WARN_NSUP_RESET.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_reset (ViSession instrSession)
{
    ViStatus rssmv_status = VI_SUCCESS;

    /*  Initialize the instrument to a known state.  */
    CHECKERR (viPrintf (instrSession, "*RST;*CLS\n"));
    CHECKERR (rssmv_defaultInstrSetup (instrSession));
        
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result. If the self test function is not supported by the   */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_SELF_TEST.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_self_test (ViSession instrSession,
                                    ViPInt16 testResult,
                                    ViChar _VI_FAR testMessage[])
{
    ViStatus rssmv_status = VI_SUCCESS;
    ViChar result1[20], result2[20], result3[20];
    ViBoolean   isPassed;

    CHECKERR (viQueryf (instrSession, ":TEST:RAM?\n", "%s", result1));
    CHECKERR (viQueryf (instrSession, ":TEST:ROM?\n", "%s", result2));
    CHECKERR (viQueryf (instrSession, ":TEST:BATT?\n", "%s", result3));
    
    isPassed = (!strstr (result1, "Passed") ||
                !strstr (result2, "Passed") ||
                !strstr (result3, "O.K."));

    if (testResult)
         *testResult = (!isPassed) ? 0 : (-1);
    
    if (testMessage)
        sprintf(testMessage, "Self-Test %s.",
            (!isPassed) ? "Passed" : "Failed");
    
    CHECKERR (rssmv_checkStatus (instrSession));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result. If the error query function is not supported by the */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_ERROR_QUERY.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_error_query (ViSession instrSession,
                                      ViPInt32 errorCode,
                                      ViChar _VI_FAR errorMessage[])
{
    ViStatus rssmv_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, ":SYST:ERR?\n"));
    CHECKERR (viScanf (instrSession, "%ld,\"%[^\"]", errorCode, errorMessage));

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_error_message (ViSession instrSession,
                                        ViStatus statusCode,
                                        ViChar _VI_FAR errMessage[])
{
    ViStatus rssmv_status = VI_SUCCESS;
    ViInt16 i;

    static rssmv_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,     "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,        "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,    "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,  "WARNING: Error Query not supported"},     
        {VI_WARN_NSUP_REV_QUERY,    "WARNING: Revision Query not supported"},
        {VI_ERROR_PARAMETER1,   "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,   "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,   "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,   "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,   "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,   "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,   "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,   "ERROR: Parameter 8 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,"ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE, "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,    "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE,   "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_INSTR_PARAMETER9 ,  "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_PARAMETER10,  "ERROR: Parameter 10 out of range"},
        {VI_ERROR_INSTR_PARAMETER11,  "ERROR: Parameter 11 out of range"},
        {VI_ERROR_INSTR_PARAMETER12,  "ERROR: Parameter 12 out of range"},
        {VI_ERROR_INSTR_PARAMETER13,  "ERROR: Parameter 13 out of range"},
        {VI_ERROR_INSTR_PARAMETER14,  "ERROR: Parameter 14 out of range"},
        {VI_ERROR_INSTR_PARAMETER15,  "ERROR: Parameter 15 out of range"},

        {RSSMV_ERROR_INSTRUMENT_ERROR,      "ERROR: Instrument status error"},
        {RSSMV_ERROR_INVALID_CONFIGURATION, "ERROR: Instrument configuration error"},
        {RSSMV_ERROR_INSTRUMENT_OPTION,     "ERROR: Option not installed"},
        {RSSMV_ERROR_INSTRUMENT_NSUP_MODEL, "ERROR: Required Instrument model is not available."},
        {VI_NULL, VI_NULL}
    };

    rssmv_status = viStatusDesc (instrSession, statusCode, errMessage);
    if (rssmv_status == VI_WARN_UNKNOWN_STATUS)
        {
        for (i = 0; statusDescArray[i].stringName; i++)
            {
            if (statusDescArray[i].stringVal == statusCode)
                {
                sprintf (errMessage, "%s", statusDescArray[i].stringName);
                return (VI_SUCCESS);
                }
            }
        sprintf (errMessage, "Unknown Error 0x%X", statusCode);
        return (VI_WARN_UNKNOWN_STATUS);
        }
    
    rssmv_status = VI_SUCCESS;
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*           If the revision query function is not supported by the          */ 
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_REV_QUERY.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_revision_query (ViSession instrSession,
                    ViChar _VI_FAR driverRev[], ViChar _VI_FAR instrRev[])
{
    ViStatus rssmv_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, "*IDN?\n"));
    CHECKERR (viScanf (instrSession, "%*[^,],%*[^,],%*[^,],%[^\r\n]", instrRev));

    sprintf (driverRev, "%s", RSSMV_REVISION);
    
    return rssmv_status;
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmv_close (ViSession instrSession)
{
    rssmv_instrRange instrPtr;
    ViSession rmSession;
    ViStatus rssmv_status = VI_SUCCESS;

    CHECKERR( viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession));
    CHECKERR( viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    if (instrPtr != NULL) 
        free (instrPtr);

    rssmv_status = viClose (instrSession);
    viClose (rmSession);

    return rssmv_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean rssmv_invalidViBooleanRange (ViBoolean val)
{
    return (ViBoolean)((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rssmv_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean rssmv_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Check String                                                    */
/* Purpose:  This function checks a string value to see if it is             */
/*           of a defined length and initialized.                            */
/*===========================================================================*/
ViBoolean rssmv_invalidViString (ViString val, ViUInt32 maxLength)
{
    if (val)
        return (ViBoolean)((strlen (val) > maxLength) ? VI_TRUE : VI_FALSE);

    return VI_TRUE;
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the rssmv_init function.  When   */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from rssmv_init to       */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus rssmv_initCleanUp (ViSession openRMSession,
                             ViPSession openInstrSession,
                             ViStatus currentStatus)
{
    rssmv_instrRange instrPtr;
    
    if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
        if (instrPtr != NULL) 
            free (instrPtr);

    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
}

/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the rssmv_reset operation and by the      */
/*           rssmv_init function if the reset option has not been            */
/*           selected. This function is useful for configuring any           */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/                                    
/*===========================================================================*/
ViStatus rssmv_defaultInstrSetup (ViSession instrSession)
{
    ViStatus    rssmv_status = VI_SUCCESS;
    ViChar      instrModel[BUFFER_SIZE];
    rssmv_instrRange instrPtr;

    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    
    if (instrPtr == NULL) 
        instrPtr = malloc (sizeof (struct rssmv_statusDataRanges));

    instrPtr -> instrOption = 0;
    instrPtr -> instrModel  = 0;

    /* Check instrument options */
    CHECKERR (rssmv_instrOptions (instrSession, &(instrPtr -> instrOption)));

    instrPtr -> errorChecking = VI_TRUE;
    strcpy (instrPtr -> instrDriverRevision, RSSMV_REVISION);

    CHECKERR (viQueryf (instrSession, "*IDN?\n", "%*[^,],%[^,],%*[^,],%*[^\r\n]",
                        instrModel));

    /* Check for instrument model */
    {
    ViInt32     i = 0;
    ViString    instrList[] =
        {
        "SMV01",
        "SMV02",
        "SMV03",

        VI_NULL
        };

    for ( i = 0; instrList[i]; i++ )
        if (strstr (instrModel, instrList[i]))
            {
            instrPtr -> instrModel = (1 << i);
            break;
            }
    }

    CHECKERR (viSetAttribute (instrSession, VI_ATTR_USER_DATA, (ViUInt32)instrPtr));

    return rssmv_status;
}

/*=========================================================================*/
/* Function: Check Status                                                  */
/* Purpose:  This function reads the system status byte.                   */
/*=========================================================================*/
ViStatus rssmv_checkStatus (ViSession instrSession)
{
    ViStatus    rssmv_status = VI_SUCCESS;
    ViInt32     esrByte = 0;
    rssmv_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    if (instrPtr -> errorChecking)
        { /* Is Error Checking Enabled? */
        /*--------------- Read the Standard Event Register ------------------*/
        CHECKERR (viQueryf (instrSession, "*ESR?\n", "%ld", &esrByte));
        
        /*---------------- Check if any error or message bit was asserted ---*/
        rssmv_status = (esrByte & IEEE_ERROR_BITS) ? RSSMV_ERROR_INSTRUMENT_ERROR : VI_SUCCESS;
        }

    return rssmv_status;
}

/*===========================================================================*/
/* Function: Check Frequency Range                                           */
/* Purpose:  This function checks RF frequency to see if it lies             */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViStatus rssmv_checkFreqRange (ViSession instrSession, ViReal64 frequency, 
                                ViReal64 min, ViStatus error)
{
    ViStatus            rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;
    ViReal64            max = 3.3e9;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    if (instrPtr == VI_NULL) return VI_ERROR_INV_OBJECT;

    switch (instrPtr -> instrModel) 
        {
        case RSSMV_SMV01: max = 1.1e9; break;
        case RSSMV_SMV02: max = 2.2e9; break;
        case RSSMV_SMV03: max = 3.3e9; break;
        }

    return (frequency < min || frequency > max) ? error : VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Check the step width for the frequency setting                  */
/* Purpose:  This function checks RF frequency  step width to see if it lies */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViStatus rssmv_checkFreqStep (ViSession instrSession, ViReal64 step, 
                                ViReal64 min, ViStatus error)
{
    ViStatus            rssmv_status = VI_SUCCESS;
    rssmv_instrRange    instrPtr;
    ViReal64            max = 3.0e9;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    if (instrPtr == VI_NULL) return VI_ERROR_INV_OBJECT;

    switch (instrPtr -> instrModel) 
        {
        case RSSMV_SMV01: max = 1.0e9; break;
        case RSSMV_SMV02: max = 2.0e9; break;
        case RSSMV_SMV03: max = 3.0e9; break;
        }

    return (step < min || step > max) ? error : VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Instrument Options                                              */
/* Purpose:  This function returns a list of options which the instrument is */
/*           equipped with.                                                  */
/*===========================================================================*/
ViStatus rssmv_instrOptions (ViSession instrSession, ViPInt32 options)
{                              
    ViStatus    rssmv_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      option[BUFFER_SIZE],
                *p2option;
    ViUInt32    i;

    ViString optionsList[] =
        {
        "B1",   /* Reference oscillator OXCO */
        "",     /* reserved */
        "B3",   /* Pulse modulation and pulse generator */
        "",     /* reserved */
        "B5",   /* Stereo Coder */
        "",     /* reserved */
        "B19",  /* Rear panel connectors */

        VI_NULL
        };
    
    CHECKERR (viQueryf (instrSession, "*OPT?\n", "%[^\r\n]", buffer));
  
    *options = 0;        
    
    for (i = 0; optionsList[i]; i++)
        {
        if (strlen(optionsList[i]) == 0)
            continue;

        strcpy(option, optionsList[i]);
        p2option = buffer;

        while ((p2option = strstr(p2option, option)) != VI_NULL)
            {
            /* the last option */
            if (strchr(p2option, ',') && !strchr(option, ','))
                strcat(option, ",");

            if (!strncmp(p2option, option, strlen(option)))
                {
                *options |= (1 << i);
                break;
                }
            p2option++;
            }
        }

    return rssmv_status;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
